part of '../../git_assistant.dart';

/// The type of stdout parser in [BaseGitChain.exec]
typedef GitStdoutParser<T> = T Function(String);

/// A parent for all classes which are commands or options.
///
/// Provides some functionalities which are common in every command and option.
abstract class BaseGitChain {
  /// Base constructor.
  ///
  /// [chainName] is required for generating result command to execute.
  ///
  /// [prevChain] is optional because the root chain has no previous.
  BaseGitChain(this.chainName, [this.prevChain]);

  /// The current chain name, which is used in generating command in [argList].
  ///
  /// This property is also required because all chains should have name.
  @protected
  final String chainName;

  /// Refers to the previous chain of the current chain that use in [argList] to generate
  /// command sequence.
  @protected
  final BaseGitChain? prevChain;

  /// Generates command sequence and returns it as `List<String>` to use in
  /// [exec], [toString] and other functionalities in subclasses.
  @protected
  List<String> get argList {
    var list = <String>[];

    list.addAll([
      ...?prevChain?.argList,
      chainName,
    ]);

    return list;
  }

  /// Executes the generated command in the [argList] and returns the result.
  ///
  /// Some subclasses use parser to parse the stdout of the given sequence command
  /// because of this the output type is generic.
  T exec<T>({GitStdoutParser<T>? parser}) {
    // TODO: Provide default parser [boolean flag]
    if (parser != null) {
      return parser(_executor() as String);
    }
    return _executor();
  }

  dynamic _executor() {
    var args = argList;
    var procRes = Process.runSync(args.removeAt(0), args);
    return procRes.stdout;
  }

  @override
  String toString() {
    return argList.join(' ');
  }
}
